from typing import Literal

WorkCardMovementType = Literal["ARRIVAL", "DEPARTURE"]

LateDeclarationJustificationType = Literal[
    "POWER_OUTAGE", "EMPLOYER_SYSTEMS_UNAVAILABLE", "ERGANI_SYSTEMS_UNAVAILABLE"
]

OvertimeJustificationType = Literal[
    "ACCIDENT_PREVENTION_OR_DAMAGE_RESTORATION",
    "URGENT_SEASONAL_TASKS",
    "EXCEPTIONAL_WORKLOAD",
    "SUPPLEMENTARY_TASKS",
    "LOST_HOURS_SUDDEN_CAUSES",
    "LOST_HOURS_OFFICIAL_HOLIDAYS",
    "LOST_HOURS_WEATHER_CONDITIONS",
    "EMERGENCY_CLOSURE_DAY",
    "NON_WORKDAY_TASKS",
]

ScheduleWorkType = Literal["WORK_FROM_OFFICE", "WORK_FROM_HOME", "REST_DAY", "ABSENT"]
